#!/bin/sh

# uvspec tools directory
tools="../../tools/"

# number of wavelength intervals
intervals=15

# filter functions usually drop rapidly to the left and right and can 
# be neglected below some limit; determine left and right limits 
# so that the loss is smaller than $accur
accur=0.001

# space-separated list of filter functions
filter="ch039 ch062 ch073 ch087 ch097 ch108 ch120 ch134"

# wavelength unit in the filter function, nm or micron
filter_unit=micron

#########################################
# don't change anything below this line #
#########################################

for flt in $filter; do

  echo ... working on filter function $flt

  if `test $filter_unit = "micron"`; then
    echo ... converting $flt from micron to wavenumbers
    gawk '!/#/{print 1e4/$1,$2+0}' $flt | sort -g > $flt.wvn
  fi

  if `test $filter_unit = "nm"`; then
    echo ... converting $flt from nm to wavenumbers
    gawk '!/#/{print 1e7/$1,$2+0}' $flt | sort -g > $flt.wvn
  fi

  echo ... integrating weighted irradiance
  gawk 'BEGIN{counter=1;sum[1]=0};FNR>1{++counter;sum[counter]=sum[counter-1]+0.5*($2+y)*($1-x);};{wvl[counter]=$1;x=$1;y=$2};END{for (i=1;i<=counter;i++) printf("%s %.8e\n", wvl[i],sum[i]/sum[counter])}' $flt.wvn > $flt.int

  echo ... determining lower and upper limits to reach accuracy $accur
  lower=`gawk 'BEGIN{accur='$accur'/2};{wvl[FNR]=$1};$2<accur{lower=FNR};END{print wvl[lower]}' $flt.int`
  upper=`gawk 'BEGIN{accur='$accur'/2};{wvl[FNR]=$1};$2<1-accur{upper=FNR};END{upper++;print wvl[upper]}' $flt.int`

  echo ... selected wavelength range $lower to $upper cm-1

  gawk 'BEGIN{lower='$lower';upper='$upper';n='$intervals';step=(upper-lower)/n;for(i=0;i<n;i++) {print "'$tools'/integrate -q -l -a"lower+i*step" -b"lower+(i+1)*step" '$flt.wvn'";printf("%.8g\n", lower+(i+0.5)*step)>"filter.x"}}' > tmp00001.sh

  bandwidth=`gawk 'BEGIN{print ('$upper'-'$lower')/'$intervals'}'`
  echo ... wavenumber step $bandwidth cm-1
  
  echo ... creating filter function file $flt.trm
  sh tmp00001.sh | gawk 'BEGIN{while ((getline < "filter.x") > 0) wvn[++counter]=$1};{printf("%.6g %.12g\n", 1e7/wvn[FNR],$1/'$bandwidth')}' | sort -g > $flt.trm

  echo ... creating thermal bands file $flt.bands
  gawk 'BEGIN{lower='$lower';upper='$upper';n='$intervals';step=(upper-lower)/n;for(i=0;i<n;i++) printf("%.6g %.12g %.12g\n", 1e7/(lower+(i+0.5)*step),  1e7/(lower+(i+1)*step), 1e7/(lower+i*step))}' | sort -g > $flt.bands

  echo ... creating thermal bands file $flt.wvl
  gawk 'BEGIN{lower='$lower';upper='$upper';n='$intervals';step=(upper-lower)/n;for(i=0;i<n;i++) printf("%.6g %.12g %.12g\n", 1e7/(lower+(i+0.5)*step)/1000.0, 1e7/(lower+(i+1)*step)/1000.0, 1e7/(lower+i*step)/1000.0)}' | sort -g > $flt.wvl

#  rm -f $flt.ext $flt.int $flt.wvn $flt.spl $flt.tmp $flt.x tmp00001.sh filter.x
  echo
done





