typedef struct scadis_struct {
  double z_sc_cc;
  double y_sc_cc;
  double mu_uda_db;
  double dirold_dx[3];
  double d_phi;
  double epsfac;

  /* probably not obsolete */
  double mu_max;
  double mu_min;
  double F_max[99], F_min[99];

} scadis_struct;

/* Initialization functions */

static inline int mc_vroom_check_and_verbose (sample_struct *sample, int quiet,
					      int cloud_phases);

static inline int mc_vroom_prepare (sample_struct *sample, atmosphere_struct *atmos,
				    cld3d_out_struct *wcld3d, cld3d_out_struct *icld3d,
				    pft *phase_aer, int nonHGaer, int nlyr, int quiet);

/* functions called by photon journey */

static inline int mc_vroom_cloning (photon_struct *p,
				    atmosphere_struct *atmos,
				    sample_struct *sample,
				    result_struct *result,
				    elevation_struct *elev,
				    albedo_struct *albedo,
				    surftemp_struct *surftemp,
				    int *absorption, int *source, 
				    int plotpath, int visualize,
				    int nonHGaer,
				    phase_function_table *phase_cld,
				    phase_function_table *phase_ice,
				    pft *phase_aer,
				    float *wvnmlo, float *wvnmhi,
				    float bplkavg,
				    float *refind,
				    float r_earth,
				    int quiet);

static inline int mc_vroom_splitting_and_rr (photon_struct *p,
					     atmosphere_struct *atmos,
					     sample_struct *sample,
					     result_struct *result,
					     elevation_struct *elev,
					     albedo_struct *albedo,
					     surftemp_struct *surftemp,
					     int *absorption, int *source, 
					     int plotpath, int visualize,
					     int nonHGaer,
					     phase_function_table *phase_cld,
					     phase_function_table *phase_ice,
					     pft *phase_aer,
					     float *wvnmlo, float *wvnmhi,
					     float bplkavg,
					     float *refind,
					     float r_earth,
					     int quiet);

double get_phase_max (sample_struct *sample, double mu2, int SC_mode);

/* helper functions */

static void cp_locest (locest_struct *dest, locest_struct *source,
                       sample_struct *sample);

/* scattering functions */

static inline void init_scadis (scadis_struct *scadis, int nphasemax);

static inline int mc_vroom_prep_DDIS (sample_struct *sample, photon_struct *p,
				      atmosphere_struct *atmos,
				      int *DDISsing, int *FODDISsing, int *out_of_cone,
				      locest_struct *lest, scadis_struct *scadis);

static inline int mu_scatter_special (atmosphere_struct *atmos, photon_struct *p,
				      phase_function_table *phase_cld,
				      int nonHGaer, pft *phase_aer,
				      phase_function_table *phase_ice,
				      pft **phase_max, int n_phase_max,
				      int ist, double *mu, scadis_struct scadis,
				      int DDISsing, int out_of_cone);

static inline int random_reflection_special (sample_struct *sample,
					     atmosphere_struct *atmos, 
					     elevation_struct *elev,
					     photon_struct *p,
					     pft **phase_max, int n_phase_max,
					     double *mu, double *phi,
					     scadis_struct *scadis, locest_struct lest,
					     int DDISsing, int FODDISsing, int out_of_cone);

static int mu_DDIS_UPF_scatter (pft **phase_max, int n_phase_max,
				scadis_struct scadis,
				int out_of_cone, int scaled, double *mu);

static int inline mu_DDIS_UDA_scatter (double mucut, double *mu);

static inline int mc_vroom_set_mus_and_phis (sample_struct *sample, photon_struct *p,
					     int DDISsing,
					     locest_struct lest, scadis_struct scadis,
					     double *mu, double *mu2, double *stheta2,
					     double phi, double *cphi2);

static inline int mc_vroom_scattering_calc_phases_and_jacobians (
                  sample_struct *sample, photon_struct *p,
		  atmosphere_struct *atmos,
		  int nonHGaer, pft *phase_aer,
		  phase_function_table *phase_cld,
		  phase_function_table *phase_ice,
		  double mu1,
		  double *P_norm, double *P_spec,
		  double *P_isoene);

static inline int mc_vroom_DDIS_weight_and_prep_stuff (
                  sample_struct *sample, photon_struct *p,
		  atmosphere_struct *atmos,
		  double mu2, double stheta2, double cphi2,
		  int DDISsing, int out_of_cone,
		  double P_norm, double P_spec, double P_isoene,
		  locest_struct lest, scadis_struct scadis);

static double calc_P_spec (double P_norm, double mu2,
			   int out_of_cone, int behind_detector,
			   double eps_ddis_upf,
			   pft **phase_max, int n_phase_max,
			   double eps_fod_dis_phi,
			   double cphi2,
			   double eps_ddis_uda,
			   scadis_struct scadis,
			   int scaled);
